<?php

/**
 * @title        Actions Log - Messages
 * @copyright    Copyright (C) 2011-2021 Minitek, All rights reserved.
 * @license      GNU General Public License version 3 or later.
 * @author url   https://www.minitek.gr/
 * @developers   Minitek.gr / Yannis Maragos
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\Component\Actionlogs\Administrator\Helper\ActionlogsHelper;

/**
 * Minitek System Messages - Actions Log Plugin.
 *
 * @since  4.0.1
 */
class plgSystemActionslogMessages extends JPlugin
{
	/**
	 * Application object.
	 *
	 * @var    JApplicationCms
	 * @since  4.0.1
	 */
	var $app;

	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  4.0.1
	 */
	protected $autoloadLanguage = true;

	/**
	 * Constructor.
	 *
	 * @param   object  &$subject  The object to observe.
	 * @param   array   $config    An optional associative array of configuration settings.
	 *
	 * @since   4.0.1
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		$this->app = Factory::getApplication();
		$this->user = Factory::getUser();

		$actionsLogAuthorized = array_intersect($this->user->getAuthorisedViewLevels(), $this->params->get('actions_log_access', [], 'ARRAY'));
		$this->user_authorized = ($this->app->isClient('site') && $actionsLogAuthorized) ||
			($this->app->isClient('administrator') && $this->params->get('enable_backend', false) && $actionsLogAuthorized);
	}

	/**
	 * After Initialise Event.
	 *
	 * @return  void
	 *
	 * @since   4.0.1
	 */
	public function onAfterInitialise()
	{
		// Reset last id if not eventsource request
		if (
			!($this->app->input->get('group', '') === 'system' &&
			$this->app->input->get('plugin', '') === 'miniteksystemmessages' &&
			$this->app->input->get('type', '') === 'serverSentEvent')
		) {
			if (is_null($this->app->getUserState('plgSystemMinitekSystemMessages.lastActionsLogId'))) 
				$this->app->setUserState('plgSystemMinitekSystemMessages.lastActionsLogId', 0);
		}
	}

	/**
	 * Plugin event to be triggered by Minitek System Messages component.
	 *
	 * @return  array
	 *
	 * @since   4.0.1
	 */
	function onGetMsmPlugins()
	{
		$plugin = array(
			'name' => 'actionslogmessages',
			'title' => Text::_('PLG_SYSTEM_ACTIONSLOGMESSAGES'),
			'image' => '',
			'user_authorized' => $this->user_authorized
		);

		return $plugin;
	}

	/**
	 * Get plugin messages.
	 *
	 * @return   void
	 *
	 * @since   4.0.1
	 */
	public function onGetMsmPluginMessages()
	{
		$messages = array();
		
		// Check for unauthorized access
		if ($this->user_authorized) 
		{
			// Initialize user state variable for a new session
			if (is_null($this->app->getUserState('plgSystemMinitekSystemMessages.lastActionsLogId')) ||
				!$this->app->getUserState('plgSystemMinitekSystemMessages.lastActionsLogId')) 
			{
				// Get id of most recent action in log
				$row = $this->getActionsLog(1);
				$id = ($row && isset($row[0])) ? (int)$row[0]->id : 0;

				$this->app->setUserState('plgSystemMinitekSystemMessages.lastActionsLogId', $id);
			}
			else 
			{
				// Get last id
				$lastId = $this->app->getUserState('plgSystemMinitekSystemMessages.lastActionsLogId');
					
				// Search for new records after last id
				$rows = $this->getActionsLog(false, $lastId);
				$found = $rows && isset($rows[0]->id);

				if ($found) 
				{
					foreach ($rows as $key => $row) 
					{
						$messages[$key]['text'] = $row->message;
						$messages[$key]['type'] = 'info';
					}
					
					// Update last Id with first record in array (array is reversed, so first record is last action log)
					$this->app->setUserState('plgSystemMinitekSystemMessages.lastActionsLogId', $rows[0]->id);
				}
			}
		}
		
		if (!empty($messages))
			return $messages;

		return false;
	}

	/**
	 * Process extension names.
	 *
	 * @param   string   $extension  Extension name.
	 *
	 * @return  string
	 *
	 * @since   4.0.1
	 */
	public static function processExtensionNames($extension)
	{
		if ($extension == 'com_plugins')
			$extension = 'com_plugins.plugin';

		return $extension;
	}

	/**
	 * Get a list of actions log records.
	 *
	 * @param   int  $count		The number of records to retrieve.
	 * @param   int  $id  		Get records after this id.
	 *
	 * @return  mixed  An array of records, or false on error.
	 *
	 * @since   4.0.1
	 */
	public function getActionsLog($count = false, $id = false)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('a.*, u.name')
			->from('#__action_logs AS a')
			->leftJoin('#__users AS u ON a.user_id = u.id');

		if ($id)
			$query->where('a.id > ' . $id);

		// Filter by extensions
		$extensions = $this->params->get('actions_log_extensions', [], 'ARRAY');

		$comma_separated = false;

		if (!empty($extensions) && !in_array('all', $extensions)) {
			$extensions = array_map('static::processExtensionNames', $extensions);
			$comma_separated = implode("','", $extensions);
			$comma_separated = "'" . $comma_separated . "'";
		}

		if ($comma_separated)
			$query->where($db->quoteName('a.extension') . ' IN (' . $comma_separated . ')');

		$query->order('a.id DESC');
		$db->setQuery($query, 0, $count);
		$rows = $db->loadObjectList();

		// Load all actionlog plugins language files
		ActionlogsHelper::loadActionLogPluginsLanguage();

		// Load plg_user_terms language file
		$lang = Factory::getLanguage();
		$lang->load('plg_user_terms', JPATH_ADMINISTRATOR);

		foreach ($rows as $row) {
			$row->message = static::getHumanReadableLogMessage($row);
		}

		return $rows;
	}

	/**
	 * Get human readable log message for a User Action Log.
	 *
	 * @param   stdClass  $log            A User Action log message record
	 * @param   boolean   $generateLinks  Flag to disable link generation when creating a message
	 *
	 * @return  string
	 *
	 * @since   3.0.1
	 */
	public static function getHumanReadableLogMessage($log)
	{
		static $links = array();

		$message = Text::_($log->message_language_key);

		// Remove links from formatted message
		$message = str_replace('<a href="{accountlink}">{username}</a>', '<b>{username}</b>', $message);
		$message = str_replace("<a href='{accountlink}'>{username}</a>", '<b>{username}</b>', $message);
		$message = str_replace('<a href="{itemlink}">{title}</a>', '<b>{title}</b>', $message);
		$message = str_replace("<a href='{itemlink}'>{title}</a>", '<b>{title}</b>', $message);

		$messageData = json_decode($log->message, true);

		// Special handling for translation extension name
		if (isset($messageData['extension_name'])) {
			ActionlogsHelper::loadTranslationFiles($messageData['extension_name']);
			$messageData['extension_name'] = Text::_($messageData['extension_name']);
		}

		// Translating application
		if (isset($messageData['app']))
			$messageData['app'] = Text::_($messageData['app']);

		// Translating type
		if (isset($messageData['type']))
			$messageData['type'] = Text::_($messageData['type']);

		$linkMode = Factory::getApplication()->get('force_ssl', 0) >= 1 ? JRoute::TLS_FORCE : JRoute::TLS_IGNORE;

		foreach ($messageData as $key => $value) {
			$message = str_replace('{' . $key . '}', $value, $message);
		}

		return $message;
	}
}
