<?php

/**
 * @title        FAQ Book Pro - Messages
 * @copyright    Copyright (C) 2011-2021 Minitek, All rights reserved.
 * @license      GNU General Public License version 3 or later.
 * @author url   https://www.minitek.gr/
 * @developers   Minitek.gr / Yannis Maragos
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\Component\FAQBookPro\Site\Helper\RouteHelper;

/**
 * Minitek System Messages - FAQ Book Pro Plugin.
 *
 * @since  4.0.1
 */
class plgSystemFaqbookproMessages extends JPlugin
{
	/**
	 * Application object.
	 *
	 * @var    JApplicationCms
	 * @since  4.0.1
	 */
	var $app;

	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  4.0.1
	 */
	protected $autoloadLanguage = true;

	/**
	 * Constructor.
	 *
	 * @param   object  &$subject  The object to observe.
	 * @param   array   $config    An optional associative array of configuration settings.
	 *
	 * @since   4.0.1
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		$this->app = Factory::getApplication();
		$this->user = Factory::getUser();

		$this->user_authorized = $this->user->id &&
			(($this->app->isClient('site') &&
			array_intersect($this->user->getAuthorisedViewLevels(), $this->params->get('fbp_answers_access', [], 'ARRAY'))) ||
			($this->app->isClient('administrator') && $this->params->get('enable_backend', false) &&
			array_intersect($this->user->getAuthorisedViewLevels(), $this->params->get('fbp_answers_access', [], 'ARRAY'))));
	}

	/**
	 * After Initialise Event.
	 *
	 * @return  void
	 *
	 * @since   4.0.1
	 */
	public function onAfterInitialise()
	{
		// Reset last id if not eventsource request
		if (
			!($this->app->input->get('group', '') === 'system' &&
			$this->app->input->get('plugin', '') === 'miniteksystemmessages' &&
			$this->app->input->get('type', '') === 'serverSentEvent')
		) {
			$this->app->setUserState('plgSystemMinitekSystemMessages.lastFbpAnswerId', 0);
		}
	}

	/**
	 * Plugin event to be triggered by Minitek System Messages component.
	 *
	 * @return  array
	 *
	 * @since   4.0.1
	 */
	function onGetMsmPlugins()
	{
		$plugin = array(
			'name' => 'faqbookpromessages',
			'title' => Text::_('PLG_SYSTEM_FAQBOOKPROMESSAGES'),
			'image' => '',
			'user_authorized' => $this->user_authorized
		);

		return $plugin;
	}

	/**
	 * Get plugin messages.
	 *
	 * @return   void
	 *
	 * @since   4.0.1
	 */
	public function onGetMsmPluginMessages()
	{
		$messages = array();

		// Check for unauthorized access
		if ($this->user_authorized) 
		{
			if ($rows = $this->getFbpAnswers())
			{
				foreach ($rows as $key => $row) 
				{
					$messages[$key]['text'] = $row->message;
					$messages[$key]['type'] = 'info';
				}
			}
		}
		
		if (!empty($messages))
			return $messages;

		return false;
	}

	/**
	 * Get FAQ Book Pro answers.
	 *
	 * @return   void
	 *
	 * @since   4.0.1
	 */
	public function getFbpAnswers()
	{
		if (is_null($this->app->getUserState('plgSystemMinitekSystemMessages.lastFbpAnswerId'))
			|| !$this->app->getUserState('plgSystemMinitekSystemMessages.lastFbpAnswerId')) 
		{
			$this->app->setUserState('plgSystemMinitekSystemMessages.lastFbpAnswerId', 0);
		}
	
		$lastId = $this->app->getUserState('plgSystemMinitekSystemMessages.lastFbpAnswerId');
		$answers = $this->getAnswers($this->user->id, $lastId);

		if (!$answers)
			return;

		foreach ($answers as $answer)
		{
			$questionUrl = Route::_(RouteHelper::getQuestionRoute($answer->q_id, $answer->q_topicid));
			$questionUrl = str_replace('administrator/', '', $questionUrl);
			$answerUrl = $questionUrl . '#a' . $answer->id;
			$text = Text::sprintf('PLG_SYSTEM_FAQBOOKPROMESSAGES_NEW_ANSWER', $answerUrl);
			$text .= ' <a href="' . $questionUrl . '">' . $answer->q_title . '</a>.';

			$answer->message = $text;
		}

		$this->app->setUserState('plgSystemMinitekSystemMessages.lastFbpAnswerId', $answers[count($answers) - 1]->id);
	
		return $answers;
	}

	/**
	 * Get the unseen answers.
	 *
	 * @param   int  $user	The id of the question author.
	 * @param   int  $id  Get next records after this id.
	 *
	 * @return  mixed  An array of records, or false on error.
	 *
	 * @since   4.0.1
	 */
	public function getAnswers($user, $id = false)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('a.id, q.id as q_id, q.title as q_title, q.topicid as q_topicid')
			->from('#__minitek_faqbook_answers AS a')
			->leftjoin('#__minitek_faqbook_questions AS q ON q.id = a.question_id')
			->where('q.created_by = ' . $user)
			->where('a.created_by != ' . $user)
			->where('a.state = 1')
			->where('a.seen = 0');

		if ($id)
			$query->where('a.id > ' . $id);

		$query->order('a.id ASC');
		$db->setQuery($query);
		$rows = $db->loadObjectList();

		return $rows;
	}
}
