<?php

/**
 * @title        Minitek System Messages
 * @copyright    Copyright (C) 2011-2021 Minitek, All rights reserved.
 * @license      GNU General Public License version 3 or later.
 * @author url   https://www.minitek.gr/
 * @developers   Minitek.gr / Yannis Maragos
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Uri\Uri;

/**
 * Minitek System Messages Plugin.
 *
 * @since  3.0.1
 */
class plgSystemMinitekSystemMessages extends JPlugin
{
	/**
	 * Buffer output padding in bytes.
	 *
	 * @since  4.0.7
	 */
	const OUTPUT_PADDING = 2048;
		
	/**
	 * Application object.
	 *
	 * @var    JApplicationCms
	 * @since  3.0.1
	 */
	var $app;

	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  3.0.1
	 */
	protected $autoloadLanguage = true;

	/**
	 * Constructor.
	 *
	 * @param   object  &$subject  The object to observe.
	 * @param   array   $config    An optional associative array of configuration settings.
	 *
	 * @since   3.0.1
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		$this->app = Factory::getApplication();
		$this->user = Factory::getUser();
	}

	/**
	 * After Initialise Event.
	 * Checks for ajax requests and server-sent events.
	 *
	 * @return  void
	 *
	 * @since   3.0.1
	 */
	public function onAfterInitialise()
	{
		$jinput = $this->app->input;

		// Check session expiration
		if (
			$jinput->get('group', '') === 'system' &&
			$jinput->get('plugin', '') === 'miniteksystemmessages' &&
			$jinput->get('type', '') === 'checkSession'
		) {
			if (!$this->user->id) {
				$session_message = array(
					array('message' => Text::_('PLG_SYSTEM_MINITEKSYSTEMMESSAGES_USER_SESSION_EXPIRED'), 'type' => 'warning')
				);

				jexit(json_encode($session_message));
			} else {
				jexit(false);
			}
		}

		// Get real-time messages
		if (
			$jinput->get('group', '') === 'system' &&
			$jinput->get('plugin', '') === 'miniteksystemmessages' &&
			$jinput->get('type', '') === 'serverSentEvent'
		) {
			Session::checkToken('get') or jexit(Text::_('JINVALID_TOKEN'));

			$this->getRealTimeMessages();
		}
	}

	/**
	 * Get real-time messages.
	 *
	 * @return   void
	 *
	 * @since   4.1.0
	 */
	public function getRealTimeMessages()
	{
		// Disable default disconnect checks
		ignore_user_abort(true);

		// Set headers for stream
		header("Content-Type: text/event-stream");
		header("Cache-Control: no-cache");
		header("Access-Control-Allow-Origin: *");
		header('X-Accel-Buffering: no');

		// Time before client automatic reconnection in milliseconds
		$interval = $this->params->get('retry_interval', 5, 'INT') ? $this->params->get('retry_interval', 5, 'INT') : 5;
		$interval *= 1000;

		echo ":" . str_repeat(" ", self::OUTPUT_PADDING) . "\n";
		echo "retry: " . $interval .  "\n\n";
		
		$messages = array();
		
		// Logged users messages
		$loggedUsersAuthorized = array_intersect($this->user->getAuthorisedViewLevels(), $this->params->get('logged_users_access', [], 'ARRAY'));

		if (($this->app->isClient('site') && $this->params->get('enable_logged_users', false) && $loggedUsersAuthorized) ||
			($this->app->isClient('administrator') && $this->params->get('enable_logged_users', false) && 
			$this->params->get('enable_backend', false) && $loggedUsersAuthorized)
		) {
			$loggedUsersMessages = $this->getLoggedUsersMessages();
			$messages = array_merge($messages, $loggedUsersMessages);
		}

		// Minitek System Messages plugins messages		
		if ($pluginsMessages = (array)$this->app->triggerEvent('onGetMsmPluginMessages', array())) 
		{
			foreach ($pluginsMessages as $pluginMessages)
			{
				if (!empty($pluginMessages))
					$messages = array_merge($messages, $pluginMessages);
			}
		}

		if (!empty($messages))
		{
			$json = json_encode($messages);

			echo ":" . str_repeat(" ", self::OUTPUT_PADDING) . "\n";
			echo 'data: {"messages": ' . $json . '}';
			echo "\n\n";
		}

		ob_flush();
		flush();
	}

	/**
	 * Get logged in users message.
	 *
	 * @return   void
	 *
	 * @since   3.0.2
	 */
	public function getLoggedUsersMessages()
	{
		$messages = array();

		// Initialize user state variable
		if (is_null($this->app->getUserState('plgSystemMinitekSystemMessages.lastLoggedUsersCount'))) {
			$this->app->setUserState('plgSystemMinitekSystemMessages.lastLoggedUsersCount', 0);
		}
		else 
		{
			// Get last count
			$lastCount = $this->app->getUserState('plgSystemMinitekSystemMessages.lastLoggedUsersCount');

			// Get logged in users
			$users = $this->getUserSessions();

			// Show count if different from last count
			if (count($users) != $lastCount) 
			{
				$messages[0]['text'] = Text::_('PLG_SYSTEM_MINITEKSYSTEMMESSAGES_TOTAL_LOGGED_IN_USERS') . ' ' . count($users);
				$messages[0]['type'] = 'info';

				$this->app->setUserState('plgSystemMinitekSystemMessages.lastLoggedUsersCount', count($users));
			}
		}

		return $messages;
	}

	/**
	 * Get a list of user session records.
	 *
	 * @param   int  $count		The number of records to retrieve.
	 * @param   int  $time  	Get records after this time.
	 *
	 * @return  mixed  An array of records, or false on error.
	 *
	 * @since   3.0.2
	 */
	public function getUserSessions()
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('a.time')
			->from('#__session AS a')
			->where('a.client_id = 0')
			->where('a.userid > 0');

		$db->setQuery($query, 0);
		$rows = $db->loadObjectList();

		return $rows;
	}

	/**
	 * Before Render Event.
	 * Loads assets.
	 *
	 * @return   void
	 *
	 * @since   3.0.1
	 */
	public function onBeforeRender()
	{
		if ($this->app->isClient('site') || ($this->app->isClient('administrator') && $this->params->get('enable_backend', false))) {
			$this->loadAssets();
		}

		// Load admin css
		if ($this->app->isClient('administrator')) 
		{
			$wa = Factory::getApplication()->getDocument()->getWebAssetManager();
			$wa->registerAndUseStyle('plg_system_miniteksystemmessages', 'plg_system_miniteksystemmessages/miniteksystemmessages.css');
		}
	}

	/**
	 * Method to notify for user login success
	 *
	 * @param   array  $options  Array holding options (user, responseType)
	 *
	 * @return  void
	 *
	 * @since   3.0.1
	 */
	public function onUserAfterLogin($options)
	{
		// Login message
		if ($this->app->isClient('site') && $this->params->get('enable_login_message', false)) {
			$this->app->enqueueMessage(Text::_('PLG_SYSTEM_MINITEKSYSTEMMESSAGES_SUCCESSFUL_LOGIN'), 'message');
		}
	}

	/**
	 * Loads css and js.
	 *
	 * @return   void
	 *
	 * @since   3.0.1
	 */
	public function loadAssets()
	{
		$document = Factory::getDocument();
		$isSite = $this->app->isClient('site');
		$isAdmin = $this->app->isClient('administrator');
		$enableBackend = $this->params->get('enable_backend', false);

		// Hide system-message-container 
		$hide_container_css = '#system-message-container {
			display: none;
		}';

		$document->addStyleDeclaration($hide_container_css);

		// Session expiration
		$session_message = $this->params->get('enable_session_expiration', true) ? true : false;
		$session_redirect_link = false;

		if ($isSite && $session_redirect = $this->params->get('redirect_session_expiration', '', 'INT')) {
			$menu_item = $this->app->getMenu()->getItem($session_redirect);
			$session_redirect_link = JRoute::_($menu_item->link . '&Itemid=' . $menu_item->id, false);
		}

		// Other plugins
		$other_plugins = (array)$this->app->triggerEvent('onGetMsmPlugins', array());
		$other_plugins_authorized = false;

		if ($other_plugins)
		{
			foreach ($other_plugins as $plugin) 
			{
				if (isset($plugin['user_authorized']) && $plugin['user_authorized'])
				{
					$other_plugins_authorized = true;
					break;
				}
			}
		}

		// Logged in users
		$loggedUsersAuthorized = array_intersect($this->user->getAuthorisedViewLevels(), $this->params->get('logged_users_access', [], 'ARRAY'));

		if (($isSite && $this->params->get('enable_logged_users', false) && $loggedUsersAuthorized) ||
			($isAdmin && $this->params->get('enable_logged_users', false) && $enableBackend && $loggedUsersAuthorized)
		) {
			$logged_users = true;
		} else {
			$logged_users = false;
		}

		// Polipop options 
		$appendTo = $this->params->get('appendTo', 'body');
		$position = $this->params->get('position', 'center');
		$layout = $this->params->get('layout', 'popups');
		$theme = $this->params->get('theme', 'default');
		$icons = $this->params->get('icons', 1) ? true : false;
		$insert = $this->params->get('insert', 'before');
		$spacing = $this->params->get('spacing', 10);
		$pool = $this->params->get('pool', 0);
		$sticky = $this->params->get('sticky', 0) ? true : false;
		$progressbar = $this->params->get('progressbar', 1) ? true : false;
		$pauseOnHover = $this->params->get('pauseOnHover', 1) ? true : false;
		$closer = $this->params->get('closer', 1) ? true : false;
		$hideEmpty = $this->params->get('hideEmpty', 0) ? true : false;
		$effect = $this->params->get('effect', 'fade');
		$easing = $this->params->get('easing', 'linear');
		$effectDuration = $this->params->get('effectDuration', 250, 'INT');

		// Colors 
		$message_color = $this->params->get('message_color', '#0ec47d');
		$message_text = $this->params->get('message_text', '#ffffff');
		$message_border = $this->params->get('message_border', '#0daf71');
		$info_color = $this->params->get('info_color', '#00b1fe');
		$info_text = $this->params->get('info_text', '#ffffff');
		$info_border = $this->params->get('info_border', '#009de0');
		$warning_color = $this->params->get('warning_color', '#ffc107');
		$warning_text = $this->params->get('warning_text', '#555555');
		$warning_border = $this->params->get('warning_border', '#e0aa07');
		$error_color = $this->params->get('error_color', '#f76860');
		$error_text = $this->params->get('error_text', '#ffffff');
		$error_border = $this->params->get('error_border', '#d64f48');
		$css = '';

		if ($theme === 'default' || $theme === 'compact') {
			$css .= '
			.polipop_theme_default .polipop__notification_type_success,
			.polipop_theme_compact .polipop__notification_type_success {
				background-color: ' . $message_color . ';
				color: ' . $message_text . ';
			}
			.polipop_theme_default .polipop__notification_type_success .polipop__notification-icon svg,
			.polipop_theme_compact .polipop__notification_type_success .polipop__notification-icon svg {
				fill: ' . $message_text . ';
			}
			.polipop_theme_default .polipop__notification_type_info,
			.polipop_theme_compact .polipop__notification_type_info {
				background-color: ' . $info_color . ';
				color: ' . $info_text . ';
			}
			.polipop_theme_default .polipop__notification_type_info .polipop__notification-icon svg,
			.polipop_theme_compact .polipop__notification_type_info .polipop__notification-icon svg {
				fill: ' . $info_text . ';
			}
			.polipop_theme_default .polipop__notification_type_warning,
			.polipop_theme_compact .polipop__notification_type_warning {
				background-color: ' . $warning_color . ';
				color: ' . $warning_text . ';
			}
			.polipop_theme_default .polipop__notification_type_warning .polipop__notification-icon svg,
			.polipop_theme_compact .polipop__notification_type_warning .polipop__notification-icon svg {
				fill: ' . $warning_text . ';
			}
			.polipop_theme_default .polipop__notification_type_error,
			.polipop_theme_compact .polipop__notification_type_error {
				background-color: ' . $error_color . ';
				color: ' . $error_text . ';
			}
			.polipop_theme_default .polipop__notification_type_error .polipop__notification-icon svg,
			.polipop_theme_compact .polipop__notification_type_error .polipop__notification-icon svg {
				fill: ' . $error_text . ';
			}
			';
		} else if ($theme === 'minimal') {
			$css .= '
			.polipop_theme_minimal .polipop__notification_type_success {
				color: ' . $message_color . ';
			}
			.polipop_theme_minimal .polipop__notification_type_success .polipop__notification-icon svg {
				fill: ' . $message_color . ';
			}
			.polipop_theme_minimal .polipop__notification_type_info {
				color: ' . $info_color . ';
			}
			.polipop_theme_minimal .polipop__notification_type_info .polipop__notification-icon svg {
				fill: ' . $info_color . ';
			}
			.polipop_theme_minimal .polipop__notification_type_warning {
				color: ' . $warning_color . ';
			}
			.polipop_theme_minimal .polipop__notification_type_warning .polipop__notification-icon svg {
				fill: ' . $warning_color . ';
			}
			.polipop_theme_minimal .polipop__notification_type_error {
				color: ' . $error_color . ';
			}
			.polipop_theme_minimal .polipop__notification_type_error .polipop__notification-icon svg {
				fill: ' . $error_color . ';
			}
			';
		}

		if ($layout === 'panel') {
			$css .= '
			.polipop_layout_panel.polipop_theme_default .polipop__notification_type_success,
			.polipop_layout_panel.polipop_theme_compact .polipop__notification_type_success {
				border-color: ' . $message_border . ';
			}
			.polipop_layout_panel.polipop_theme_default .polipop__notification_type_info,
			.polipop_layout_panel.polipop_theme_compact .polipop__notification_type_info {
				border-color: ' . $info_border . ';
			}
			.polipop_layout_panel.polipop_theme_default .polipop__notification_type_warning,
			.polipop_layout_panel.polipop_theme_compact .polipop__notification_type_warning {
				border-color: ' . $warning_border . ';
			}
			.polipop_layout_panel.polipop_theme_default .polipop__notification_type_error,
			.polipop_layout_panel.polipop_theme_compact .polipop__notification_type_error {
				border-color: ' . $error_border . ';
			}
			';
		}

		$document->addStyleDeclaration($css);

		// Load Polipop
		$wa = Factory::getApplication()->getDocument()->getWebAssetManager();
		$wa->registerAndUseStyle('plg_system_miniteksystemmessages.polipop.core', 'plg_system_miniteksystemmessages/polipop.core.css');

		if ($theme === 'default')
			$wa->registerAndUseStyle('plg_system_miniteksystemmessages.polipop.default', 'plg_system_miniteksystemmessages/polipop.default.css');
		else if ($theme === 'compact')
			$wa->registerAndUseStyle('plg_system_miniteksystemmessages.polipop.default', 'plg_system_miniteksystemmessages/polipop.compact.css');
		else if ($theme === 'minimal')
			$wa->registerAndUseStyle('plg_system_miniteksystemmessages.polipop.default', 'plg_system_miniteksystemmessages/polipop.minimal.css');

		$wa->registerAndUseScript('plg_system_miniteksystemmessages.polipop', 'plg_system_miniteksystemmessages/polipop.js', [], ['defer' => true]);

		// Add script options
		$document->addScriptOptions('miniteksystemmessages', array(
			'token' => Session::getFormToken(),
			'site_path' => Uri::root(),
			'is_site' => $isSite ? true : false,
			'user_id' => $this->user->id,
			'session_message' => $session_message,
			'session_redirect_link' => $session_redirect_link,
			'lifetime' => Factory::getConfig()->get('lifetime'),
			'other_plugins' => $other_plugins_authorized,
			'logged_users' => $logged_users,
			'error_text' => Text::_('ERROR'),
			'success_text' => Text::_('MESSAGE'),
			'notice_text' => Text::_('NOTICE'),
			'warning_text' => Text::_('WARNING'),
			// Polipop options
			'appendTo' => $appendTo,
			'position' => $position,
			'layout' => $layout,
			'theme' => $theme,
			'icons' => $icons,
			'insert' => $insert,
			'spacing' => $spacing,
			'pool' => $pool,
			'sticky' => $sticky,
			'life' => $this->params->get('life', 3000, 'INT'),
			'progressbar' => $progressbar,
			'pauseOnHover' => $pauseOnHover,
			'headerText' => Text::_('PLG_SYSTEM_MINITEKSYSTEMMESSAGES_HEADER_TEXT'),
			'closer' => $closer,
			'closeText' => Text::_('PLG_SYSTEM_MINITEKSYSTEMMESSAGES_CLOSE_TEXT'),
			'loadMoreText' => Text::_('PLG_SYSTEM_MINITEKSYSTEMMESSAGES_LOAD_MORE_TEXT'),
			'hideEmpty' => $hideEmpty,
			'effect' => $effect,
			'easing' => $easing,
			'effectDuration' => $effectDuration
		));

		// Load js
		$wa->registerAndUseScript('plg_system_miniteksystemmessages', 'plg_system_miniteksystemmessages/miniteksystemmessages.js', [], ['defer' => true], ['core']);
	}
}
